==> tecfy_core_package-1.0.51.tar.gz: lib/services/notification_app.service.dart <==
part of tecfy_core_client_package;

class NotificationAppService {
  static bool connected = false;
  static IOWebSocketChannel? channel;
  static Stream<dynamic>? stream;
  static List<NotificationApp> notificationList = [];
  static int notificationCount = 0;
  static StreamController<int>? notificationCountStreamController;
  static Stream<int>? notificationCountStream;

  static init() {
    notificationCountStreamController = StreamController<int>();
    notificationCountStream =
        notificationCountStreamController!.stream.asBroadcastStream();
    notificationCount = 0;
    getLocalNotifications().then((result) {
      notificationList = result ?? [];
      notificationCount = result?.length ?? 0;
      notificationCountStreamController?.add(notificationCount);
    });
  }

  static onNotificationReceived(NotificationApp notification) {
    print('Notification Count = $notificationCount');
    if (notificationList.any((n) => n.id == notification.id)) return;
    TimeoutMessage.show(notification.title, message: notification.message);

    if (!notification.showNowOnly) {
      notificationList.insert(0, notification);
      notificationCount++;
      notificationCountStreamController?.add(notificationCount);
      saveLocal(notification);
    }
  }

  static softNotifications() {
    notificationList.sort((a, b) => b.time.compareTo(a.time));
    notificationCountStreamController?.add(notificationCount);
  }

  static loadOldNotifications() async {
    try {
      var lastMessageTime = await getLastMessageTime();
      if (lastMessageTime == null) {
        lastMessageTime = DateTime.now().toUtc().subtract(Duration(days: 30));
      }
      print(
          '+++++++++++++++++++++++ geting notification start from $lastMessageTime');
      WebSocketService.sendPackage(
        Package(
            type: PackageType.requestLatestNotification,
            data: lastMessageTime.millisecondsSinceEpoch),
      );
    } catch (err) {
      print('xxx error load old notifications : $err');
    }
  }

  static Future<DateTime?> getLastMessageTime() async {
    var result = await DatabaseService.rawQuery(
        'select time from Notification order by time desc limit 1');
    if (result == null || result.length == 0 || result.first['time'] == null)
      return null;
    return DateTime.fromMillisecondsSinceEpoch(result.first['time'] as int,
        isUtc: true);
  }

  static Future markAsRead(Notification notification) async {
    // TODO: fix the issue
  }

  static Future removeAll() async {
    notificationList = [];
    notificationCount = 0;
    notificationCountStreamController?.add(notificationCount);
    await DatabaseService.delete('Notification');
  }

  static Future saveLocal(NotificationApp notification) async {
    try {
      await DatabaseService.insert('Notification', notification.toMapDb());
    } catch (err) {
      print('XXXXXXXXXXXXXXXXXXXXXX cannot save Notification locally!!!!!!!');
      print(err);
    }
  }

  static Future saveLocalAll(List<NotificationApp> notifications) async {
    await DatabaseService.exeBatch((batch) async {
      notifications.forEach((m) => batch.insert('Notification', m.toMapDb()));
      return OperationResult(success: true);
    }, true);
  }

  static Future<List<NotificationApp>?> getLocalNotifications() async {
    var list = await DatabaseService.rawQuery(
        "select * from Notification order by time desc");
    return list?.map((m) => NotificationApp.fromMap(m)).toList();
  }
}

==> tecfy_core_package-1.1.19.tar.gz: lib/services/notification_app.service.dart <==
part of tecfy_core_client_package;

class NotificationAppService {
  static bool connected = false;
  static IOWebSocketChannel? channel;
  static Stream<dynamic>? stream;
  static List<NotificationApp> notificationList = [];
  static int notificationCount = 0;
  static StreamController<int>? notificationCountStreamController;
  static Stream<int>? notificationCountStream;

  static init() async {
    notificationCountStreamController = StreamController<int>();
    notificationCountStream =
        notificationCountStreamController!.stream.asBroadcastStream();
    notificationCount = 0;
    getLocalNotifications().then((result) {
      notificationList = result ?? [];
      notificationCount = result?.length ?? 0;
      notificationCountStreamController?.add(notificationCount);
    });

    FirebaseMessaging.onMessageOpenedApp.listen((msg) {
      print('------------- onMessageOpenedApp---------- ');
      print(msg.messageType);
      print(msg.contentAvailable);
      print(msg.data);
    });
    FirebaseMessaging.onMessage.listen((msg) {
      print('------------- massge---------- ');
      onNotificationReceived(NotificationApp(
        id: msg.messageId ?? '',
        time: msg.sentTime ?? DateTime.now(),
        type: 0,
        seen: false,
        ownerUserId: LocalConfigService.user?.id ?? '',
        titleAr: msg.notification?.title ?? '-',
        titleEn: msg.notification?.title ?? '-',
        messageEn: msg.notification?.body ?? '-',
        messageAr: msg.notification?.body ?? '-',
      ));
    });

    //await FirebaseMessaging.instance.subscribeToTopic('weather');
    // var msgs = await messaging.getInitialMessage();
    // if (msgs != null) {

    // }
  }

  static onNotificationReceived(NotificationApp notification) {
    print('Notification Count = $notificationCount');
    if (!notification.showNowOnly &&
        notificationList.any((n) => n.id == notification.id)) return;

    TimeoutMessage.show(notification.title, message: notification.message);

    if (!notification.showNowOnly) {
      notificationList.insert(0, notification);
      notificationCount++;
      notificationCountStreamController?.add(notificationCount);
      saveLocal(notification);
    }
  }

  static softNotifications() {
    notificationList.sort((a, b) => b.time.compareTo(a.time));
    notificationCountStreamController?.add(notificationCount);
  }

  static loadOldNotifications() async {
    try {
      var lastMessageTime = await getLastMessageTime();
      if (lastMessageTime == null) {
        lastMessageTime = DateTime.now().toUtc().subtract(Duration(days: 30));
      }
      print(
          '+++++++++++++++++++++++ geting notification start from $lastMessageTime');
      WebSocketService.sendPackage(
        Package(
            type: PackageType.requestLatestNotification,
            data: lastMessageTime.millisecondsSinceEpoch),
      );
    } catch (err) {
      print('xxx error load old notifications : $err');
    }
  }

  static Future<DateTime?> getLastMessageTime() async {
    var result = await DatabaseService.rawQuery(
        'select time from Notification order by time desc limit 1');
    if (result == null || result.length == 0 || result.first['time'] == null)
      return null;
    return DateTime.fromMillisecondsSinceEpoch(result.first['time'] as int,
        isUtc: true);
  }

  static Future markAsRead(Notification notification) async {
    // TODO: fix the issue
  }

  static Future removeAll() async {
    notificationList = [];
    notificationCount = 0;
    notificationCountStreamController?.add(notificationCount);
    await DatabaseService.delete('Notification');
  }

  static Future saveLocal(NotificationApp notification) async {
    try {
      var json = notification.toMapDb();
      print(json);
      await DatabaseService.insert('Notification', json);
    } catch (err) {
      print('XXXXXXXXXXXXXXXXXXXXXX cannot save Notification locally!!!!!!!');
      print(err);
    }
  }

  static Future saveLocalAll(List<NotificationApp> notifications) async {
    await DatabaseService.exeBatch((batch) async {
      notifications.forEach((m) => batch.insert('Notification', m.toMapDb()));
      return OperationResult(success: true);
    }, true);
  }

  static Future<List<NotificationApp>?> getLocalNotifications() async {
    var list = await DatabaseService.rawQuery(
        "select * from Notification order by time desc");
    return list?.map((m) => NotificationApp.fromMap(m)).toList();
  }
}
